#!/bin/bash

if [[ -z "$CUMBIA_INSTALL_ROOT" ]]; then
    echo -e "[\e[1;31mNO\e[0m]: CUMBIA_INSTALL_ROOT not exported. Please"
    echo -e "export CUMBIA_INSTALL_ROOT=path/to/cumbia-libs"
    exit 1
fi

# Configuration file path
CONFIG_FILE="$HOME/.config/cumbia-project-tools/user.conf"
TEMPLATE_CONFIG="$CUMBIA_INSTALL_ROOT/share/cumbia-project-tools/conf/user.conf"

# Variables from config file
declare -a dev_dirs
make_command=""
cmake_conf=""

# Function to load the configuration file
load_config() {
    if [[ -f "$CONFIG_FILE" ]]; then
        source "$CONFIG_FILE"
        if [[ ${#dev_dirs[@]} -eq 0 ]]; then
            echo -e "[\e[1;31mNO\e[0m]: missing \"dev_dirs\" (${#dev_dirs[@]})  directive in '$CONFIG_FILE'"
            exit 1
        fi
        if [[ -z "$make_command" ]]; then
            echo -e "[\e[1;31mNO\e[0m]: missing \"make_command\" directive in '$CONFIG_FILE'"
            exit 1
        fi
        if [[ -z "$cmake_conf" ]]; then
            echo -e "[\e[1;31mNO\e[0m]: missing \"cmake_conf\" directive in '$CONFIG_FILE'"
            exit 1
        fi
    else
        echo "Configuration file not found at $CONFIG_FILE."
        echo "Please run the script with the -i option to initialize a draft configuration file."
        exit 1
    fi
}

initialize_config() {
    if [[ -f "$CONFIG_FILE" ]]; then
        if [[ "$force_overwrite_conf" == true ]]; then
            cp "$TEMPLATE_CONFIG" "$CONFIG_FILE" \
            && echo "Configuration file overwritten at $CONFIG_FILE" \
            && echo "Please personalize the file before continuing"
        else
            echo "Configuration file already exists at $CONFIG_FILE"
            echo "Use -f to force overwrite."
            exit 1
        fi
    else
        if [[ -f "$TEMPLATE_CONFIG" ]]; then
            mkdir -p "$(dirname "$CONFIG_FILE")"
            cp "$TEMPLATE_CONFIG" "$CONFIG_FILE" \
            && echo "Configuration file initialized at $CONFIG_FILE" \
            && echo "Please personalize the file before continuing"
        else
            echo "Configuration source file not found: $TEMPLATE_CONFIG"
            exit 1
        fi
    fi
    exit 0
}

check_gitlab_config() {
    if [[ ${#gitlab_groups_search[@]} -eq 0 ]]; then
        echo -e "add gitlab_groups_search=(\"cs/gui\" \"cs/lib\" \"cs/lib/cumbia\")"
        echo -e "line into $CONFIG_FILE"
        echo -e "if you want to search for the project in gitlab"
        exit 1
    fi
    if [[ -z "$gitlab_token" ]]; then
        echo -e "add gitlab_token=your-gitlab-personal-access-token"
        echo -e "line into $CONFIG_FILE"
        echo -e "if you want to search for the project in gitlab"
        exit 1
    fi
    if [[ -z "$gitlab_url" ]]; then
        echo -e "add gitlab_url=\"https://gitlab.elettra.eu\""
        echo -e "line into $CONFIG_FILE"
        echo -e "if you want to search for the project in gitlab"
        exit 1
    fi
    if ! which jq > /dev/null 2>&1; then
        echo -e "[\e[1;31mNO\e[0m]: jq is not installed. Please install jq"
        echo -e "if you want to search for the project in gitlab"
    exit 1
fi
}
# Function to list all development directories (-L)
list_dev_dirs() {
    echo "Development directories:"
    for dir in "${dev_dirs[@]}"; do
        echo "$dir"
    done
}

# Function to list projects with optional search (-l with -s)
list_projects() {
    if [[ -z "$dir_leaf" ]]; then
        # No specific directory leaf provided, list projects from all configured directories
        echo "Listing projects from all configured directories:"
        for dir in "${dev_dirs[@]}"; do
            echo "Projects in $dir:"
            for proj in "$dir"/*; do
                if [[ -d "$proj" ]]; then
                    proj_name=$(basename "$proj")
                    if [[ -n "$search_str" && "$proj_name" != *"$search_str"* ]]; then
                        continue
                    fi

                    if [[ -f "$proj/CMakeLists.txt" ]]; then
                        echo -e "$proj_name [\e[1;32mcmake\e[0m]"
                    elif [[ -n $(find "$proj" -maxdepth 1 -name '*.pro') ]]; then
                        echo -e "$proj_name [\e[1;36mqmake\e[0m]"
                    else
                        echo -e "$proj_name [\e[1;35munknown\e[0m]"
                    fi
                fi
            done
        done
    else
        # Specific directory leaf provided, list projects from the matched directory
        local target_dir=""
        for dir in "${dev_dirs[@]}"; do
            if [[ "$(basename "$dir")" == "$dir_leaf" ]]; then
                target_dir="$dir"
                break
            fi
        done

        if [[ -z "$target_dir" ]]; then
            echo "Directory leaf '$dir_leaf' not found."
            exit 1
        fi

        echo "Projects in $target_dir:"
        for proj in "$target_dir"/*; do
            if [[ -d "$proj" ]]; then
                proj_name=$(basename "$proj")
                if [[ -n "$search_str" && "$proj_name" != *"$search_str"* ]]; then
                    continue
                fi

                if [[ -f "$proj/CMakeLists.txt" ]]; then
                    echo "$proj_name [\e[1;32mcmake\e[0m]"
                elif [[ -n $(find "$proj" -maxdepth 1 -name '*.pro') ]]; then
                    echo "$proj_name [\e[1;36mqmake\e[0m]"
                else
                    echo "$proj_name [\e[1;35munknown\e[0m]"
                fi
            fi
        done
    fi
}

# Function to get the group ID from the GitLab group name
get_group_id() {
    local group_name="$1"

    # Fetch the group information using the GitLab API
    group_info=$(curl -s --header "PRIVATE-TOKEN: $gitlab_token" "$gitlab_url/api/v4/groups?search=$group_name")
    # Extract the group ID using jq
    group_id=$(echo "$group_info" |jq -r --arg full_path "$group_name" '.[] | select(.full_path == $full_path) | .id')

    if [[ -z "$group_id"  ]] || [[ "$group_id" == "null" ]]; then
        echo "Error: Group '$group_name' not found."
        return 1
    fi

    echo "$group_id"
}

gitlab_group_search() {
    for group in "${gitlab_groups_search[@]}"; do
        local project="$1"
        group_id=$(get_group_id "$group")
        if [[ $? -ne 0 ]]; then
            echo -e "[\e[1;31mNO\e[0m]: Failed to retrieve group ID."
            exit 1
        fi

        # Esegui una richiesta per ottenere i progetti del gruppo
        # curl --header "PRIVATE-TOKEN: <your_access_token>" \
        # "https://gitlab.example.com/api/v4/groups/3/   \
        # search?scope=projects&search=flight"
        projects=$(curl --silent --header "PRIVATE-TOKEN: $gitlab_token" \
            "$gitlab_url/api/v4/groups/$group_id/search?scope=projects&search=$project")

        # Filtra i progetti usando jq per cercare il progetto
        project_url=$(echo "$projects" | jq -r --arg name $project '.[] | select(.name == $name) | .http_url_to_repo')


        if [[ -n "$project_url" ]]; then
            echo -e "found \e[1;32m$project\e[0m at \e[1;3m$project_url\e[0m\n"
            if [[ -n "$make" || -n "$configure" || -n "$clean" || -n "$rebuild" ]]; then
                echo "Cloning project from GitLab: $project_url"
                # Usa la directory di output se specificata, altrimenti usa la directory corrente
                git clone "$project_url" "${output_dir:-.}/$project_name"
                echo "Project cloned successfully."
            fi
            break
        fi
    done
}

gitlab_search() {
    for group in "${gitlab_groups_search[@]}"; do

        group_id=$(get_group_id "$group")

        if [[ $? -eq 0 ]]; then
            echo "Group ID for '$group' is: $group_id"
        else
            echo "gitlab_search: failed to retrieve group ID."
            exit 1
        fi

        # Esegui una richiesta per ottenere i progetti del gruppo
#        projects=$(curl --silent --header "PRIVATE-TOKEN: $gitlab_token" \
#            "$gitlab_url/api/v4/groups/$group/projects?per_page=100")

#        # Filtra i progetti usando jq per cercare il progetto
#        project_url=$(echo "$projects" | jq -r --arg name "$project_name" '.[] | select(.name == $name) | .http_url_to_repo')

#        if [[ -n "$project_url" ]]; then
#            if [[ -n "$make" || -n "$configure" || -n "$clean" || -n "$rebuild" ]]; then
#                echo "Cloning project from GitLab: $project_url"
#                # Usa la directory di output se specificata, altrimenti usa la directory corrente
#                git clone "$project_url" "${output_dir:-.}/$project_name"
#                echo "Project cloned successfully."
#            fi
#            break
#        fi
    done
}

# Function to determine project type (-t)
get_project_type() {
    if [[ -f "$project_path/CMakeLists.txt" ]]; then
        echo "cmake"
    elif [[ -n $(find "$project_path" -maxdepth 1 -name '*.pro') ]]; then
        echo "qmake"
    else
        echo "Unknown project type."
        exit 1
    fi
}

# Function to clean a project (-x)
clean_project() {
    local project_type=$(get_project_type)
    if [[ "$project_type" == "cmake" ]]; then
        rm -rf "$project_path/build"
        echo "Cleaned CMake project: $project_path"
    elif [[ "$project_type" == "qmake" ]]; then
        cd "$project_path" && make distclean
        echo "Cleaned QMake project: $project_path"
    fi
}

# Function to configure a project (-c)
configure_project() {
    local project_type=$(get_project_type)
    if [[ "$project_type" == "cmake" ]]; then
        mkdir -p "$project_path/build" && cd "$project_path/build" && eval "$cmake_conf"
        echo "Configured CMake project: $project_path"
    elif [[ "$project_type" == "qmake" ]]; then
        cd "$project_path" && qmake
        echo "Configured QMake project: $project_path"
    fi
}

# Function to build a project (-m)
make_project() {
    local project_type=$(get_project_type)
    if [[ "$project_type" == "cmake" ]]; then
        if [[ -d "$project_path/build" ]]; then
            cd "$project_path/build" && eval "$make_command"
            if [[ $? -eq 0 ]]; then
                echo -e "[\e[1;32mOK\e[0m]: built CMake project: $project_path"
            else
                echo -e "[\e[1;31mNO\e[0m]: failed executing '$make_command' from '$project_path'"
            fi
        else
            echo -e "[\e[1;31mNO\e[0m]: '\e[1;33mbuild/\e[0m' directory missing within a cmake project."
            echo "execute $0 with -c before -m"
        fi
    elif [[ "$project_type" == "qmake" ]]; then
        cd "$project_path" && eval "$make_command"
        if [[ $? -eq 0 ]]; then
            echo "[\e[1;32mOK\e[0m]: built QMake project: $project_path"
        else
            echo "[\e[1;31mNO\e[0m]: build failed into $project_path"
            echo "[\e[1;31mNO\e[0m]: make sure qmake has been run (execute $0 again with -c)"
        fi
    fi
}

# Function to rebuild a project (-r)
rebuild_project() {
    clean_project
    configure_project
    make_project
}

# Parse command-line options
while getopts "iLls:d:p:mcxrtf" opt; do
    case ${opt} in
        i )
            initialize_config_opt=true
            ;;
        L )
            load_config
            list_dev_dirs
            ;;
        l )
            list=true
            ;;
        s )
            search_str=$OPTARG
            ;;
        d )
            dir_leaf=$OPTARG
            ;;
        p )
            project_name=$OPTARG
            ;;
        m )
            make_opt=true
            ;;
        c )
            configure_opt=true
            ;;
        x )
            clean_opt=true
            ;;
        r )
            rebuild_opt=true
            ;;
        t )
            type_opt=true
            ;;
        f )
            force_overwrite_conf=true
            ;;
        o ) output_dir=$OPTARG
            ;;  # output dir

        \? )
            echo "Usage: $0 [-i] [-lL] [-d dir] [-p project] [-m] [-c] [-x] [-r] [-t] [-s search] [ -o clone-output-dir]"
            exit 1
            ;;
    esac
done

if [[ "$initialize_config_opt" == true ]]; then
    echo "initializing config"
    initialize_config
fi

# Load configuration if not initializing
if [[ "$list" == true || -n "$project_name" ]]; then
    load_config
fi

# List all projects in a development directory (-l)
if [[ "$list" == true ]]; then
    list_projects
fi

# Handle project-based options (-p)
if [[ -n "$project_name" ]]; then
    for dir in "${dev_dirs[@]}"; do
        if [[ -d "$dir/$project_name" ]]; then
            project_path="$dir/$project_name"
            break
        fi
    done

    if [[ -z "$project_path" ]]; then
        echo -e "Project $project_name not found. Searching \e[1;32mgitlab\e[0m..."
        # Controllo se il progetto esiste
        # Utilizza gitlab_url, gitlab_token, e gitlab_groups_search per cercare il progetto
        check_gitlab_config
        gitlab_group_search $project_name

        # Se non è stato trovato alcun progetto
        if [[ -z "$project_url" ]]; then
            echo "Project '$project_name' not found in specified GitLab groups."
            exit 1
        fi

        exit 1
    fi

    if [[ "$type_opt" == true ]]; then
        get_project_type
    elif [[ -z "$rebuild_opt" ]] || [[ "$rebuild_opt" == false ]] ; then
        if [[ "$clean_opt" == true ]]; then
            clean_project
        fi
        if [[ "$configure_opt" == true ]]; then
            configure_project
        fi
        if [[ "$make_opt" == true ]]; then
            make_project
        fi
    elif [[ "$rebuild_opt" == true ]]; then
        rebuild_project
    fi
fi
