#!/bin/bash

source cumbia-project-search-functions.sh
source cumbia-project-init-conf-functions.sh

# Variables from config file
declare -a dev_dirs
make_command=""
cmake_conf=""

group_from_cwd() {
    # we are in group/project_name dir
    parentd="${PWD%/*}"  # This removes the last section after the last /
    # see if this dir or parent ends with one of the groups
    local devdc=""
    dirs=("$PWD" "$(dirname "$PWD")")
    for dir in "${dirs[@]}"; do
        if [[ -z "$devdc" ]]; then
            for devd in "${gitlab_groups_search[@]}"; do
                # Remove trailing slash if necessary
                devdc="${devd%/}"
                if [[ "$dir" == *"$devdc" ]]; then
                    break;
                else
                    devdc=""
                fi
            done
        fi
    done
    echo "$devdc"
}

find_matching_group() {
    project_url="$1"
    # Step 1: Remove the last section (project)
    parent_url="${project_url%/*}"  # This removes the last section after the last /
    # Step 2: Extract the part after the domain to match against the array
    # This removes the 'https://gitlab.eu/' part
    # gitlab_url%/ removes trailing '/' if present
    remaining_url="${parent_url#${gitlab_url%/}/}"
    # Step 3: Compare the remaining path with each element in the gitlab_groups_search array
    local grpc=""
    for grp in "${gitlab_groups_search[@]}"; do
        # Remove trailing slash if necessary
        grpc="${grp%/}"
        if [[ "$grpc" == *"$remaining_url" ]]; then
            break;
        else
            grpc=""
        fi
    done
    echo $grpc
}

find_devdir_by_group() {
    group="$1"
    group="${group%/}"
    for devdir in "${dev_dirs[@]}"; do
        if [[   "$devdir" == *"$group" ]]; then
            echo "$devdir"
            break
        fi
    done
    echo ""
}

gather_git_log_msgs_since_last_tag() {
    # Check if there is at least one tag in the repository
    if [[ -n get_last_tag ]]; then
        # Get the log messages since the last tag
        echo "$(eval "$git_get_log_since_last_tag_cmd")"
    fi
}

get_last_tag() {
    lt=$(eval "$git_get_last_tag_cmd")
    echo "$lt"
}

urlencode() {
    local raw="${1}"
    local encoded=""
    for (( i=0; i<${#raw}; i++ )); do
        local c="${raw:$i:1}"
        case "$c" in
            [-_.~a-zA-Z0-9] ) encoded+="$c" ;;
            * ) encoded+=$(printf '%%%02X' "'$c")
        esac
    done
    echo "$encoded"
}

get_last_tag_from_remote_url() {
    project_url="$(urlencode "$1")"
    project_id=$(curl --silent --header "PRIVATE-TOKEN: $gitlab_token" "$gitlab_url/api/v4/projects/$project_url" | jq -r '.id')
    if [[ -n "$project_id" ]]; then
        tag=$(curl --silent --header "PRIVATE-TOKEN: $gitlab_token" "$gitlab_url/api/v4/projects/$project_id/repository/tags" | jq -r '.[0].name')
    fi
    echo "$tag"
}

# Function to increment the version of the last tag
get_new_tag() {
    latest_tag="$1"
    if [ -z "$latest_tag" ]; then
        echo "No tags found. Please create an initial tag."
        return 1
    fi
    # Remove the 'v' prefix if present
    version="${latest_tag#v}"
    # Split the version into its components
    IFS='.' read -r major minor patch <<< "$version"
    # Increment the patch version
    patch=$((patch + 1))
    # Form the new version
    new_version="$major.$minor.$patch"
    # Create a new tag with the incremented version
    echo "$new_version"
}

release() {
    message="$1"
    project="$2"
    tag="$3"

    # Check for uncommitted changes in the working directory (ignores untracked files)
    if [[ -n $(git status --porcelain | grep -v "??") ]]; then
        echo -e "[ \e[1;31m:-(\e[0m ]: there are uncommitted changes in the working directory (showing tracked files only)."
        git status --short | grep -v "??"
        exit 1
    fi

    # Check if the local branch is ahead of the remote branch
    git fetch

    # Check if there are any unpushed commits
    if [[ -n $(git log @{u}..HEAD 2>/dev/null) ]]; then
        echo -e "[ \e[1;31m:-(\e[0m ]: there are unpushed commits."
        git log --oneline @{u}..HEAD
        echo -e "[ \e[1;31m:-(\e[0m ]: try the -s (sync) option first"
        exit 1
    fi

    # Check for unpulled commits (remote commits not in local)
    unpulled_commits=$(git rev-list --count HEAD..HEAD@{u})
    if [[ $unpulled_commits -gt 0 ]]; then
        echo -e "[ \e[1;31m:-(\e[0m ]: there are $unpulled_commits unpulled commits (you might need to pull)."
        git log --oneline HEAD..HEAD@{u}
        exit 1
    fi

    last_tag=$(get_last_tag)

    if [[ -z "$tag" ]]; then
        # no tag provided as $3 argument
        if [[ -z "$last_tag" ]]; then
            echo -e "[ \e[1;33m:-o\e[0m ]: no previous tags"
            echo -e "[ \e[1;33m:-o\e[0m ]: using tag '1.0.0'"
            tag="1.0.0"
            if [[ -z "$message" ]]; then
                message="first release of the project"
                echo  -e "[ \e[1;33m:-o\e[0m ]: no previous tags and no message provided: using '$message' as message"
            fi
        else
            tag=$(get_new_tag $last_tag)
            if [[ -z "$message" ]]; then
                message="$(gather_git_log_msgs_since_last_tag)"
                if [[ -n "$message" ]]; then
                    echo -e "[ \e[1;33m:-o\e[0m ]: message not provided: will release with the following message:"
                    echo -e "[ \e[1;33m:-o\e[0m ]: \e[1;3m$message\e[0m\n"
                fi
            fi
        fi
    fi

    if [[ -z "$message" ]]; then
        message="$(git log -1 --pretty=format:%s)"
    fi
    if [[ -n "$message" ]]; then
        if [[ "$execute" == true ]]; then
            echo -e "\e[1;36mRELEASE\e[0m calling \e[1;3mgit tag -a $tag -m \"$message\"\e[0m\n"
            git tag -a $tag -m "$message"
            eval "$git_push_tags_cmd"
            exit 0
        else
            echo -e "[ \e[1;36mRELEASE\e[0m ]: \e[1;33mwould\e[0m call"
            echo -e " \e[1;3mgit tag -a $tag -m \""$message"\"\e[0m"
            echo -e " \e[1;3m$git_push_tags_cmd\e[0m"
            echo -e "[ \e[1;36mRELEASE\e[0m ]: call with \e[1;31m-x\e[0m to execute\n"
            exit 0
        fi
    else
        echo -e "[ \e[1;31m:-(\e[0m ]: a message was not provided and no log messages could be"
        echo -e "[ \e[1;31m:-(\e[0m ]: gathered since last tag \e[1;3m$last_tag\e[0m"
        echo -e "[ \e[1;31m:-(\e[0m ]: call $0 with -m \"a release message\""
        exit 1
    fi
}

create() {
    project=$1
    group=$2
    dire=$(basename $PWD)
    if [[ -z "$project" ]] then
        project=$PWD
        echo -e "[ \e[0;32mCREATE\e[0m ]: project name \e[1;3m$project\e[0m guessed from current directory"
    else
        echo -e "[ \e[0;32mCREATE\e[0m ]: project name \e[1;3m$project\e[0m"
    fi
    if [[ -z "$group" ]]; then
        # guess group from working dir
        group=$(group_from_cwd)
        echo -e "[ \e[0;32mCREATE\e[0m ]: group name \e[1;3m$group\e[0m guessed from current directory"
    else
        echo -e "[ \e[0;32mCREATE\e[0m ]: group name \e[1;3m$group\e[0m"
    fi
    if [[ -z "$group" ]]; then
        echo -e "[ \e[1;33mCREATE\e[0m ]: unable to guess group from current dir $PWD use $0 -g my/group"
        echo -e "[ \e[1;33mCREATE\e[0m ]: alternatively change directory in one of"
        dirs=("$PWD" "$(dirname "$PWD")")
        for dir in "${gitlab_groups_search[@]}"; do
            echo -e "         $dir"
        done
        echo -e "[ \e[1;33mCREATE\e[0m ]: development (sub) directories configured in the $CONFIG_FILE (\"gitlab_groups_search\")"
        echo -e "\n[ \e[1;33mCREATE\e[0m ]: if you continue, the project will be created under your $gitlab_url \"home\""
        echo -e "[ \e[1;33mCREATE\e[0m ]: do you want to continue [y|n]?"
        read response
        if [[ $response != "y" ]]; then
            echo -e "[ \e[1;33mCREATE\e[0m ]: operation cancelled"
            exit 0
        fi
    fi
    if [[ -z "$description" ]]; then
        description="$project application"
    fi
    echo -e "[ \e[0;32mCREATE\e[0m ]: description: \e[1;3m$description\e[0m"

    if [[ -n "$group" ]]; then
        namespace_id=$(curl -s --header "PRIVATE-TOKEN: $gitlab_token" "$gitlab_url/api/v4/groups?search=$group" | jq '.[0].id')
        if [[ -n "$namespace_id" ]] && [[ -n "$project" ]] && [[ -n "$gitlab_url" ]] ; then
            cmd="curl --silent --request POST --header \"PRIVATE-TOKEN: $gitlab_token\" \
                 --header \"Content-Type: application/json\" --data '{ \"name\": \"$project\", \"description\": \"$description\", \
                \"namespace_id\": \"$namespace_id\", \"visibility\": \"public\" }' \
                --url \"$gitlab_url/api/v4/projects/\""
        fi
    elif [[ -z "$group" ]] && [[ -n "$project" ]] && [[ -n "$gitlab_url" ]] ; then
        cmd="curl --silent --request POST --header \"PRIVATE-TOKEN: $gitlab_token\" \
            --header \"Content-Type: application/json\" --data '{ \"name\": \"$project\", \"description\": \"$description\", \
            \"visibility\": \"public\" }' \
            --url \"$gitlab_url/api/v4/projects/\""
    else
        echo -e "[ \e[1;31mCREATE\e[0m ]: error in project name '$project' gitlab url '$gitlab_url' or failed to get namespace_id: $namespace_id"
        exit 1
    fi

    echo -e "\e[1;33m$cmd\e[0m"
    jsres="$(eval "$cmd")"
    echo -e "returned $jsres\n"
    msg=$(echo "$jsres" | jq '.message')
    if [[ -z "$msg" ]] || [[ "$msg" == "null" ]]; then
        msg=$(echo "$jsres" | jq '.error')
    fi
    if [[ -n "$msg" ]] && [[ "$msg" != "null" ]]; then
        echo -e "[ \e[1;31m:-(\e[0m ]: received \e[1;3m$msg\e[0m"
        exit 1
    fi
    arr=($(echo "$jsres" | jq '.id, .name, .web_url, .created_at'))
    echo -e "[ \e[0;32mCREATE\e[0m ]: created \e[1;3m${arr[0]}\e[0m URL: \e[0;3m${arr[1]}\e[0m at $"
}

synchronize() {
    project_name="$1"
    group="$2"
    message="$3"
    execute="$4"
    BRANCH="master"  # Change to your branch (main, master, etc.)

    # Check if the current directory is a Git repository
    if [ ! -d ".git" ]; then
        if [[ "$execute" == true ]]; then
            echo -e "[ \e[1;36mSYNC\e[0m ]: initializing new git repository..."

            if  [[ -n "$group" ]]; then
                origin=$gitlab_url/$group/$project_name.git
            else
                origin=$gitlab_url/$gitlab_user/$project_name.git
            fi

            git init --initial-branch=$BRANCH

            if [[ $? -eq 0 ]]; then
                git remote add origin $origin
            fi
            if [[ $? -eq 0 ]]; then
                git add .
            fi
            if [[ $? -eq 0 ]]; then
                git commit -m "$project_name initial commit"
            fi
            if [[ $? -eq 0 ]]; then
                git push --set-upstream origin $BRANCH
            fi
        else
            echo  -e "[ \e[1;36mSYNC\e[0m ]: \e[1;33;3mwould initialize\e[0m new git repository."
            echo  -e "[ \e[1;36mSYNC\e[0m ]: call with \e[1;31m-x\e[0m to execute\n"
        fi
    else
        if [[ -n $(git status --porcelain | grep -v "??") ]]; then
            echo -e "[ \e[1;36mSYNC\e[0m ]: there are uncommitted changes in the working directory. committing"
            git status --short | grep -v "??"
            if [[ -z "$message" ]]; then
                echo -e "[ \e[1;36mSYNC\e[0m ]: provide a message with \e[1;33m-m\e[0m to commit changes"
                exit 1
            elif [[ "$execute" == true ]] && [[ -n "$message" ]]; then
                echo -e "[ \e[1;36mSYNC\e[0m ]: \e[1;32mcommitting changes\e[0m..."
                git commit -a -m "$message"
            elif [[ -n "$message" ]]; then
                echo -e "[ \e[1;36mSYNC\e[0m ]: \e[1;33mwould\e[0m call \e[1;3mgit commit -a -m \""$message"\"\e[0m"
                echo -e "[ \e[1;36mSYNC\e[0m ]: call with \e[1;31m-x\e[0m to execute\n"
                exit 0
            fi
        fi

        echo -e "[ \e[1;36mSYNC\e[0m ]: fetching latest changes..."
        # Fetch the latest changes from the remote repository
        git fetch origin

        if [[ ! $(git rev-parse --verify HEAD)  ]]; then
            echo -e "[ \e[1;31mSYNC\e[0m ]: probably there are no commits yet"
            exit 1
        fi

        # Get the local and remote branch information
        LOCAL=$(git rev-parse "$BRANCH")
        # the next requires git fetch origin first
        REMOTE=$(git rev-parse "origin/$BRANCH")
        # Get the merge base (the common ancestor) between
        # the local and remote branches.
        BASE=$(git merge-base "$BRANCH" "origin/$BRANCH")

        if [ "$LOCAL" = "$REMOTE" ]; then
            echo -e "[ \e[1;36mSYNC\e[0m ]: local and remote branches are up to date."
        elif [ "$LOCAL" = "$BASE" ]; then
            echo -e "[ \e[1;36mSYNC\e[0m ]: local branch is behind, pulling changes..."
            git pull origin "$BRANCH"
        elif [ "$REMOTE" = "$BASE" ]; then
            echo  -e "[ \e[1;36mSYNC\e[0m ]: local branch is ahead, committing and pushing changes..."
            git commit -m "Sync local changes"
            git push origin "$BRANCH"
        else
            echo -e "[ \e[1;31mSYNC\e[0m ]: local and remote have diverged. You may need to resolve conflicts manually."
            exit 1
        fi
    fi
}

