#!/bin/bash

source cumbia-project-git-functions.sh

# Variables from config file
declare -a dev_dirs
make_command=""
cmake_conf=""

# Function to load the configuration file
load_config() {
    if [[ -f "$CONFIG_FILE" ]]; then
        source "$CONFIG_FILE"
        if [[ -z "$gitlab_token" ]]; then
            echo -e "[\e[1;31m:-(\e[0m]: missing \"gitlab_token\" directive in '$CONFIG_FILE'"
            exit 1
        fi
        if [[ -z "$gitlab_url" ]]; then
            echo -e "[\e[1;31m:-(\e[0m]: missing \"gitlab_url\" directive in '$CONFIG_FILE'"
            exit 1
        fi
    else
        echo "Configuration file not found at $CONFIG_FILE."
        echo "Please run the script with the -i option to initialize a draft configuration file."
        exit 1
    fi
}

print_options() {
    action=$1
    if [[ -z "$action" ]]; then
        echo "-ggit/group(with-create) -ddescription(with-create) -c(create) -pproject_name "\
        "-u(pull) -l(log) -L(lastlog) -r(release) -mmessage " \
        "-ooutput_dir -s(sync) -n(clone) -i(init-config-file)"
    elif [[ "$action" == "sync" ]]; then
        echo "-ggit_group"
    elif [[ "$action" == "lastlog" ]]; then
        echo ""
    elif [[ "$action" == "log" ]]; then
        echo ""
    elif [[ "$action" == "release" ]]; then
        echo echo "-ggit_group -m\"release-message\""
    elif [[ "$action" == "release" ]]; then
        echo echo "-ggit_group -d\"project-description\""
    fi
}

if [[ "$initialize_config_opt" == true ]]; then
    # from source'd cumbia-project-init-conf-functions.sh
    init_config
    exit 0
fi

# load configuration.
load_config

# Parse command-line options
while getopts "p:o:m:ncg:d:ulrfiLsOxt:" opt; do
    case ${opt} in
        p )
            project=$OPTARG
            ;;
        i )
            initialize_config_opt=true
            ;;
        n )
            clone=true
            ;;
        c )
            create=true
            ;;
        g )
            group=$OPTARG
            ;;
        d )
            description="$OPTARG"
            ;;
        u )
            pull=true
            ;;
        l )
            log=true
            ;;
        L )
            lastlog=true
            ;;
        r )
            release=true
            ;;
        m)
            message="$OPTARG"
            ;;
        t)  tag=$OPTARG
            ;;
        f )
            force_overwrite_conf=true
            ;;
        o )
            output_dir=$OPTARG
            ;;
        s )
            sync=true
            ;;
        x )
            execute=true
            ;;
        O )
            print_options
            exit 0
            ;;
        \? )
            echo -e "Usage:\n\n $0 \n\n" \
                   " -p name select \e[1;32mproject\e[0m. If unspecified, assuming working dir is the project dir\n" \
                   " -n clo\e[1;32mn\e[0me a project\n" \
                   " -s \e[1;32synchronize\e[0m a project with remote, or\n" \
                   "    git init a project (that needs be -c (created) first)\n" \
                   " -c \e[1;32mcreate\e[0m a project on gitlab" \
                   "    -g \e[1;32mgroup\e[0m specify a group (by default group is guessed from current working directory)"  \
                   "    -d \e[1;32mdescription\e[0m new gitlab project description" \
                   " -o dir clone the project in the given output dir\n" \
                   "    if not specified, the ending part of the URL is used\n" \
                   "    to match one of the development dirs specified in the\n" \
                   "    configuration file.\n" \
                   "    Example \e[1;3m$0 -n pro\e[0m: project \e[1;3mpro\e[0m is searched\n" \
                   "    in gitlab. If found under https://gitlab.elettra.eu/cs/gui/pro\n" \
                   "    \e[1;4mand\e[0m \e[1;3mdev_dirs\e[0m in the configuration file contains " \
                   "\e[0;3m/home/user/devel/cs/gui\e[0m,\n" \
                   " -u \e[1;32mupdate\e[0m project with \e[1;3mgit pull\e[0m\n" \
                   " -l \e[1;32mlog\e[0m git log\n" \
                   " -L \e[1;32mLast log\e[0m logs since last tag\n" \
                   " -r \e[1;32mrelease\e[0m project (requires -t and -m)\n" \
                   "    the project is cloned there.\n\n" \
                   " -i (initialize configuration file, -f to overwrite)\n\n"
            exit 1
            ;;
    esac
done

dire=$(basename $PWD)

if [[ $dire == "$project" ]] || [[ -z "$project" ]] ; then
    in_wdir=true
    if [[ -z "$project" ]] ; then
        echo -e "[ \e[0;36mPROJECT\e[0m ]: no project name given with \e[1;3m-p\e[0m: working " \
        "in current directory: \e[1;3m$dire\e[0m"
        project=$dire
    fi
fi

if [[ -z "$group" ]] && [[ "$in_wdir" == true ]]; then
    group=$(group_from_cwd)
    if [[ -z "$group" ]]; then
        echo -e "[ \e[0;33mGROUP\e[0m ]: failed to determine group from current dir \e[1;3m$PWD\e[0m and \e[1;3m-g group\e[0m not specified"
    else
        echo -e "[ \e[0;36mGROUP\e[0m ]: \e[1;3m$group\e[0m from current dir"
    fi
fi

# see if a remote project exists
exact_match=1
if gitlab_groups_enabled; then
    echo "searching in gitlab groups porject $project and group $group"
    json_remote_pro=$(gitlab_group_search "$project" "$group")
    echo "json_extract on $json_remote_pro"
    remote_pro=$(json_extract  "$json_remote_pro" "$project" "$exact_match")
fi

# remote projects found in any group? if not, search under user's profile repo
if [[ ${#remote_pro[@]} -eq 0 ]] || [[ -z "${remote_pro[0]}" ]]; then
    echo -e "[ \e[0;36mPROJECT\e[0m ]: \e[1;3m$project\e[0m not found across gitlab groups. searching in user's repo"
    json_remote_pro=$(gitlab_search "$project")
    remote_pro=$(json_extract  "$json_remote_pro" "$project" "$exact_match")
    if [[ ${#remote_pro[@]} -eq 1 ]] && [[ -n "${remote_pro[0]}" ]]; then
        pro_in_user_repo=true
    fi
fi

# is project on gitlab?
if [[ ${#remote_pro[@]} -eq 1 ]] && [[ -n "${remote_pro[0]}" ]]; then
    pro_in_gitlab=true
    remote_grp=$(find_matching_group "${remote_pro[0]}")
    # remote_grp will be empty if project is under the user's main page
    # if -p project specified (not in wdir) and no group specified
    # assign group to remote_grp
    if [[ -z "$group" ]] && [[ -z "$in_wdir" ]]; then
        group=$remote_grp
    fi
    echo "remote pro ${remote_pro[0]} group $group  remote group $remote_grp"
elif [[ "$clone" == true ]] \
    || [[ "$pull" == true ]] || [[ "$log" == true ]] ||  [[ "$release" == true ]] \
    || [[ "$lastlog" == true ]]; then
    echo -e "[ \e[0;31mPROJECT\e[0m ]: \e[1;3m$project\e[0m not found on remote repositories"
    exit 1
fi

if [[ "$create" == true ]] && [[ "$pro_in_gitlab" == true ]] && [[ "$remote_grp" == "$group" ]]; then
    echo -e "[ \e[0;31mCREATE\e[0m ]: \e[1;3m$project\e[0m already in \e[1;3m${remote_pro[0]}\e[0m"
    exit 1
fi

if [[ "$create" == true ]] && [[ -z "$group" ]] && [[ -z "$pro_in_gitlab" ]]; then
    echo -e "[ \e[1;33mCREATE\e[0m ]: no group, \e[1;3m$project\e[0m not found on any repo on gitlab"
    echo -e "[ \e[1;33mCREATE\e[0m ]: project will be created under the personal page repo on gitlab"
elif [[ "$sync" == true ]]  && [[ -z "$group" ]] && [[ -z "$pro_in_gitlab" ]]; then
    echo -e "[ \e[1;31mSYNC\e[0m ]: \e[1;3m$project\e[0m not yet on gitlab: use the \e[1;3m-c\e[0m " \
        "(create) option first"
    exit 1
fi

if [[ "$sync" == true ]] && [[ "$remote_grp" != "$group" ]]; then
    echo -e "[ \e[0;31mSYNCe[0m ]: \e[1;3m$project\e[0m local group \"$group\" differs from remote group \"$remote_grp\""
    exit 1
fi

if [[ "$create" == true ]] && [[ "$execute" == true ]]; then
    create $project $group
    exit 0
fi

if [[ "$in_wdir" == true ]] && [[ "$pull" == true ]]; then
    eval "$git_pull_cmd"
elif [[ "$in_wdir" == true ]] &&  [[ "$log" == true ]]; then
    eval "$git_get_log_cmd"
elif [[ "$in_wdir" == true ]] &&  [[ "$lastlog" == true ]]; then
    eval "$gather_git_log_msgs_since_last_tag"
elif [[ "$in_wdir" == true ]] &&  [[ "$release" == true ]]; then
    release "$message" "$dire" "$tag" $execute
elif [[ "$in_wdir" == true ]] &&  [[ "$sync" == true ]]; then
    synchronize $project $group "$message" $execute
elif [[ "$pro_in_gitlab" == true ]]; then
    if [[ -z "$output_dir" ]] && gitlab_groups_enabled; then
        grp=$remote_grp
        if [[ -n "$grp" ]]; then
            output_dir=$(find_devdir_by_group $grp)
            output_dir="$output_dir/$project"
        fi
    fi
    if [[ -z "$output_dir" ]]; then
        echo -e "[ \e[1;31m:-(\e[0m ]: -o output_dir unspecified and could not determine local output dir from group \"$grp\""
        exit 1
    fi
    if [[ "$clone" ]] && [[ ! -d $output_dir ]]; then
        git clone "${remote_pro[0]}" "$output_dir"
        echo -e "[ \e[1;32m:-)\e[0m ]: project cloned in $output_dir"
    elif [[ "$pull" == true ]] && [[ -d $output_dir ]]; then
        cd $output_dir
        eval "$git_pull_cmd"
    elif [[ "$release" == true ]] && [[ -d $output_dir ]]; then
            cd $output_dir
            release "$message" "$project" "$tag" $execute
    elif  [[ "$pull" == true ]] && [[ ! -d $output_dir ]]; then
        echo -e "[ \e[1;33m:-o\e[0m ]: directory '$output_dir' does not exist"
        echo -e "[ \e[1;33m:-o\e[0m ]: use \e[1;3m$0 -n $project\e[0m to clone the project first\n"
    elif  [[ "$clone" == true ]] && [[ -d $output_dir ]]; then
        echo -e "[ \e[1;33m:-o\e[0m ]: directory '$output_dir' already exists"
        echo -e "[ \e[1;33m:-o\e[0m ]: use \e[1;3m$0 -u $project\e[0m to update sources with \e[1;3mgit pull\e[0m\n"
    elif (([[ "$log" == true ]] || [[ "$lastlog" == true ]] )  && [[ -d $output_dir ]]); then
        if [[ "$output_dir" != "$PWD" ]]; then
            cd $output_dir
            echo -e "[ \e[1;36mDIR\e[0m ] $PWD"
        fi
        if [[ "$log" == true ]]; then
            eval "$git_get_log_cmd"
        elif [[ "$lastlog" == true ]]; then
            eval "$git_get_log_since_last_tag_cmd"
        fi
    elif [[ -d $output_dir ]] && [[ "$sync" == true ]]; then
        cd "$output_dir"
        if [[ $? -eq 0 ]]; then
            echo -e "[ \e[1;36mSYNC\e[0m ] $PWD"
            synchronize "${remote_pro[0]}" $group "$message" $execute
        fi
    elif [[ ! -d $output_dir ]] && [[ "$sync" == true ]]; then
        echo -e "[ \e[1;36mSYNC\e[0m ] \e[1;3m$output_dir\e[0m does not exist: cloning project"
        git clone "${remote_pro[0]}" "$output_dir"
        if [[ $? -eq 0 ]]; then
            echo -e "[ \e[1;32mSYNC\e[0m ]: \e[1;3m${remote_pro[0]}\e[0m successfully cloned"
        else
            echo -e "[ \e[1;31mSYNC\e[0m ]: git clone failed for \e[1;3m${remote_pro[0]}\e[0m"
        fi
    elif [[  ! -d $output_dir ]]; then
        echo -e "[ \e[1;31m:-(\e[0m ]: non existent directory $output_dir"
    fi

elif  [[ ${#remote_pro[@]} -eq 0 ]]; then
    echo -e "[ \e[1;31m:-(\e[0m ]: no projects found on \e[0;3m$gitlab_url\e[0m matching \e[1;3m$project\e[0m\n"
else
    echo -e "[ \e[1;31m:-(\e[0m ]: multiple matches (${#remote_pro[@]}) found on\n" \
        "\e[0;3m$gitlab_url\e[0m matching \e[1;3m$project\e[0m. Expected only one.\n"
fi

for r in "${remote_pro[@]}"; do
    echo "$r"
done

for h in "${hints[@]}"; do
    echo -e "[ \e[1;33mHINT\e[0m ]: $h"
done

