#!/bin/bash

source cumbia-project-search-functions.sh
source cumbia-project-init-conf-functions.sh
source cumbia-project-git-functions.sh

print_options() {
    action=$1
    if [[ -z "$action" ]]; then
        echo "-pproject_name "\
        "-i(install) -x(execute!) -rtag  -ggit/group -s(search) -ffacilities"
    elif [[ "$action" == "install" ]]; then
        echo "-ggit_group -rtag -ffacilities"
    elif [[ "$action" == "search" ]]; then
        echo "-ggit_group  -ffacilities"
    fi
}

usage() {
echo -e "Usage:\n\n $0 \n\n" \
       " -p name select \e[1;32mproject\e[0m. If unspecified, assuming working dir is the project dir\n" \
       " -i \e[1;32minstall\e[0m the project (last tag if -r tag is not specified)\n" \
       " -s \e[1;32msearch\e[0m search mode, with -p search within a project history, with -f list facilities\n" \
       "    with -f facility [facility1,facility2,...] lists the hosts by facility\n" \
       " -r tag \e[1;32mrelease\e[0m with -i, install the given tag\n" \
       " -g \e[1;32mgroup\e[0m specify a group (by default group is guessed from current working directory)\n"  \
       " -f \e[1;32mfacilities\e[0m, with -i, install in the specified facilities (or facilities, comma separated) \n" \
       "\e[0;3m/home/user/devel/cs/gui\e[0m,\n\n"
}

# Parse command-line options
while getopts "p:f:g:r:isO:x" opt; do
    case ${opt} in
        p )
            project=$OPTARG
            ;;
        i )
            install=true
            ;;
        s )
            search=true
            ;;
        r )
            release="$OPTARG"
            ;;
        g )
            group=$OPTARG
            ;;
        f )
            facilities=$OPTARG
            ;;
        x )
            execute=true
            ;;
        O )
            print_options "$OPTARG"
            exit 0
            ;;
        \? )
            usage
            exit 1
            ;;
    esac
done

# Function to load the configuration file
load_config() {
    if [[ -f "$CONFIG_FILE" ]]; then
        source "$CONFIG_FILE"
    else
        echo "Configuration file not found at $CONFIG_FILE."
        echo "Please run the script \e[1;3mcumbia-project-new.sh\e[0m with the -i option to initialize a draft configuration file."
        exit 1
    fi
}

load_config

if [[ -z "$inau_host" ]] || [[ -z "$inau_user" ]]; then
    echo -e "[ \e[1;31mINAU\e[0m ]: either inau_host or inau_user (or both) configuration line(s) missing in configuration file \"$CONFIG_FILE\"\n"
    exit 1
fi

if [[ -z "$search" ]] && [[ -z "$install" ]]; then
    usage
    echo -e "[ \e[1;31mINAU\e[0m ]: either -s (search) or -i (install) are compulsory"
    exit 1
elif [[ "$search" == true ]] && [[ "$install" == true ]]; then
    usage
    echo -e "[ \e[1;31mINAU\e[0m ]: -s (search) and -i (install) are mutually exclusive"
    exit 1
fi

dire=$(basename $PWD)

if [[ -n "$project" ]]; then
    project_specified=true
fi

if [[ $dire == "$project" ]] || [[ -z "$project" ]] ; then
    in_wdir=true
    if [[ -z "$project" ]] ; then
        echo -e "[ \e[0;36mINAU\e[0m ]: no project name given with \e[1;3m-p\e[0m: examining " \
        "current directory \e[1;3m$dire\e[0m"
        project=$dire
    fi
    group=$(group_from_cwd)
fi

# find the remote project
exact_match=1
if gitlab_groups_enabled; then
    json_remote_pro=$(gitlab_group_search "$project" "$group")
else
    json_remote_pro=$(gitlab_search "$project")
fi
remote_pro=$(json_extract  "$json_remote_pro" "$project" "$exact_match")

if [[ ${#remote_pro[@]} -eq 1 ]] && [[ -n "${remote_pro[0]}" ]]; then
    pro_in_gitlab=true
fi

if [[ -z "$group" ]] && [[ "$pro_in_gitlab" == true ]] && gitlab_groups_enabled; then
    group="$(find_matching_group "${remote_pro[0]}")"
fi

if [[ -z "$group" ]] && [[ "$pro_in_gitlab" == true ]]; then
    echo -e "[ \e[1;31mINAU\e[0m ]: impossible to determine group from the current directory and -g was not used"
    exit 1
fi

if  [[ "$install" == true ]] && [[ -z "$release" ]] && [[ "$pro_in_gitlab" == true ]] ; then
    # from cumbia-project-git-functions.sh
    echo -e -n "[ \e[1;36mINAU\e[0m ]: finding last tag in \e[1;3m$group/$project\e[0m..."
    release=$(get_last_tag_from_remote_url "$group/$project")
fi

if  [[ "$install" == true ]] && ([[ -z "$release" ]] || [[ ! "$release" =~ ^[0-9]+\.[0-9]+\.[0-9]+$ ]] ); then
    echo -e "[ \e[1;31mINAU\e[0m ]: invalid release '$release'"
elif [[ "$install" == true ]] && [[ -n "$release" ]]; then
    echo -e "found \e[1;3m$release\e[0m"
fi

if [[ -z "$project" ]]; then
    echo -e "[ \e[1;31mINAU\e[0m ]: impossible to determine the project name"
    exit 1
elif [[ "$project_specified" == true ]] &&  [[ -z "$pro_in_gitlab" ]]; then
    echo -e "[ \e[1;31mINAU\e[0m ]: specified project $project not in gitlab repository"
    exit 1
fi

echo -e -n "[ \e[1;36mINAU\e[0m ]: project: \e[1;3m$project\e[0m, ";
if [[ "$pro_in_gitlab" == true ]]; then
    echo -e -n "in gitlab: \e[0;32myes\e[0m "
else
    echo -e -n "in gitlab: \e[0;35mno\e[0m "
fi
if [[ -n "$facilities" ]]; then
    echo -e -n "facilities $facilities "
fi
if [[  "$install" == true ]]; then
    echo -e -n "operation: \e[1;32minstall\e[0m"
elif [[  "$search" == true ]]; then
    echo -e -n "operation: \e[1;32msearch\e[0m"
fi
echo -e "\e[0m\n"

if [[ -z "$facilities" ]] && [[ -n "$group" ]]  && [[ -n "$project" ]]  && [[ -n "$release" ]] && [[ "$install" == true ]]; then
    # install everywhere
    # curl -k "$inau_host/v2/cs/installations" -u $inau_user -d"repository=$group/$project" -d"tag=$release"
    if [[ -z "$execute" ]]; then
        echo -e "[ \e[1;36mINAU\e[0m ]: \e[1;33;4mwould execute\e[0m: curl -k \"$inau_host/v2/cs/installations\" -u $inau_user -d\"repository=$group/$project\" " \
        "-d\"tag=$release\""
        echo -e "\n[ \e[1;36mINAU\e[0m ]: repeat with \e[1;31m-x\e[0m to execute!\n"
    else
        echo -e "[ \e[1;36mINAU\e[0m ]: curl -k \"$inau_host/v2/cs/installations\" -u $inau_user -d\"repository=$group/$project\" " \
            "-d\"tag=$release\""
        curl -k "$inau_host/v2/cs/installations" -u $inau_user -d"repository=$group/$project" -d"tag=$release"
    fi
elif [[ -n "$facilities" ]] && [[ -n "$group" ]]  && [[ -n "$project" ]]  && [[ -n "$release" ]] && [[ "$install" == true ]]; then
    # Set Internal Field Separator to comma
    IFS=',' read -ra parts <<< "$facilities"
    for f in "${parts[@]}"; do
        if [[ -z "$execute" ]]; then
            echo -e "[ \e[1;36mINAU\e[0m ]: \e[1;33;4mwould execute\e[0m: curl -k \"$inau_host/v2/cs/facilities/$f/installations\" -u $inau_user -d\"repository=$group/$project\" " \
                "-d\"tag=$release\""
            echo -e "\n[ \e[1;36mINAU\e[0m ]: repeat with \e[1;31m-x\e[0m to execute!\n"
        else
            echo -e "[ \e[1;36mINAU\e[0m ]:  curl -k \"$inau_host/v2/cs/facilities/$f/installations\" -u $inau_user -d\"repository=$group/$project\" " \
            "-d\"tag=$release\""
            curl -k "$inau_host/v2/cs/facilities/$f/installations" -u $inau_user -d"repository=$group/$project" -d"tag=$release"
        fi
    done
elif  [[ -z "$facilities" ]] && [[ "$search" == true ]] && [[ -z "$pro_in_gitlab" ]] ; then
    echo -e "[ \e[1;36mINAU\e[0m ]: facility list"
    curl -s -k $inau_host/v2/cs/facilities
elif [[ -n "$facilities" ]] && [[ "$search" == true ]] && [[ "$pro_in_gitlab" == true ]]; then
    IFS=',' read -ra parts <<< "$facilities"
    for f in "${parts[@]}"; do
        if [[ -z "$group" ]]; then
            echo -e "[ \e[1;36mINAU\e[0m ]: searching for \e[1;3m$project\e[0m installation in facility \e[1;3m$f\e[0m"
            curl -s $inau_host/v2/cs/facilities/$f/installations | grep $project
        elif  [[ -n "$group" ]]; then
            echo -e "[ \e[1;36mINAU\e[0m ]: searching for \e[1;3m$project\e[0m " \
                "installation in facility \e[1;3m$f\e[0m and group \e[1;3m$group\e[0m"
            curl -s $inau_host/v2/cs/facilities/$f/installations | grep $project | grep $group
        else
            echo -e "[ \e[1;36mINAU\e[0m ]: host list for facility \e[1;3m$f\e[0m"
            curl -k -s $inau_host/v2/cs/facilities/$f/hosts

        fi
    done
elif [[ -z "$facilities" ]] && [[ "$search" == true ]] && [[ "$pro_in_gitlab" == true ]]; then
    if [[ -n "$group" ]]; then
        curl -s $inau_host/v2/cs/installations | grep $project | grep $group
    else
        curl -s $inau_host/v2/cs/installations | grep $project
    fi
else
    echo "?"
fi


