#!/bin/bash

source cumbia-project-init-conf-functions.sh
source pycumbia-python-test.sh

if [[ -z "$CUMBIA_INSTALL_ROOT" ]]; then
    echo -e "\e[1;31mErr\e[0m: CUMBIA_INSTALL_ROOT not exported. Please"
    echo -e "export CUMBIA_INSTALL_ROOT=path/to/cumbia-libs"
    exit 1
fi

print_options() {
    echo "-nproject_name -r(remote) -ggroup -sspokesperson -ooutput_dir -f(forceoverwrite) -cclass_name -i(initconf)"
}

# Default values
output_dir="."
class_name=""
spokesperson=""
force_overwrite=false
init_config=false

# Parse command-line options
while getopts "n:c:o:s:firg:pPO" opt; do
    case ${opt} in
        n ) project_name=$OPTARG ;;
        c ) class_name=$OPTARG ;;
        o ) output_dir=$OPTARG ;;
        s ) spokesperson=$OPTARG ;;
        f ) force_overwrite=true ;;
        i ) init_config_opt=true ;;
        r ) remote=true ;;
        g ) group=$OPTARG ;;
        P ) port=true ;;
        p ) pythonpro=true ;;
        O ) print_options
            exit 0
            ;;

        \? ) echo -e "Usage: $0 -n project_name \n" \
            "[-r (remote: create the project on gitlab, commit and push the skeleton) ]\n" \
            "[-g group (remote: specify a group, like cs/gui. If unspecified, the utility\n" \
            "    \e[1;3mcumbia-project-git\e[0m tries to infer the group from the working\n" \
            "    directory: e.g. from \e[1;3m/home/user/devel/cs/gui\e[0m --> \e[1;32;3mcs/gui\e[0m) ]\n" \
            "[-c class_name]\n" \
            "[-s spokesperson]\n" \
            "[-o output_dir [-f (force overwrite) ] ]\n" \
            "[\e[1;32m-p\e[0m (generate \e[1;32mpython\e[0m code instead of C++) ]\n" \
            "[-P (Port the project in the \e[1;3mcurrent directory\e[0m from qmake to cmake ]\n" \
            "[-i (init configuration) [-f (force overwrite configuration) ] ]"
            exit 1 ;;
    esac
done

# Define template directory and destination directory
if [[ -z "$pythonpro" ]]; then
    template_dir="$CUMBIA_INSTALL_ROOT/share/cumbia-project-tools/templates/app"
else
    template_dir="$CUMBIA_INSTALL_ROOT/share/cumbia-project-tools/templates/app/python"
fi


if [[ "$init_config_opt" == true ]]; then
    init_config
fi

# Load configuration
if [[ -f "$CONFIG_FILE" ]]; then
    source "$CONFIG_FILE"
else
    echo "Configuration file not found: $CONFIG_FILE"
    echo "execute $0 -i to initialize the configuration and then edit"
    echo "$CONFIG_FILE"
    exit 1
fi

if [[ "$port" == true ]]; then
    if [[ -z "$project_name" ]]; then
        project_name=$(basename $PWD)
        if find . -maxdepth 1 -name "$project_name.pro" | grep -q . && \
            [[ ! -f CMakeLists.txt ]]; then
                cp $template_dir/CMakeLists.txt .
                sed -i "s/\${project}/$project_name/g" CMakeLists.txt
                echo -e "[ \e[1;36mCMAKE PORT\e[0m ]: CMakeLists.txt created for project $project_name in $PWD"
                echo -e `cat "$CUMBIA_INSTALL_ROOT/share/cumbia-project-tools/porting.txt"`
                exit 0
        else
            echo -e "[ \e[1;31mCMAKE PORT\e[0m ]: either $project_name.pro not found or CMakeLists.txt already present in $PWD"
            exit 1
        fi
    else
        echo -e "[ \e[1;31mCMAKE PORT\e[0m ]: sorry, this option works only from within the project directory"
        exit 1
    fi
fi

# Ensure project name is provided
if [[ -z "$project_name" ]]; then
    echo "Project name (-n) is required."
    exit 1
fi

# Ensure author name is provided
if [[ -z "$author" ]]; then
    echo "Author name is required and shall be set in the configuration file $CONFIG_FILE"
    exit 1
fi

# Set class name to camel case project name if not provided
if [[ -z "$class_name" ]]; then
    class_name=$(echo "$project_name" | sed -r 's/(^|_)([a-z])/\U\2/g')
fi

dest_dir="$output_dir/$project_name"

# Check if the destination directory exists
if [[ -d "$dest_dir" ]]; then
    if [[ "$force_overwrite" == true ]]; then
        rm -rf "$dest_dir"
    else
        echo "Error: Directory $dest_dir already exists. Use -f to overwrite."
        exit 1
    fi
fi

# Copy template directory structure to the destination
# template_dir is either $CUMBIA_INSTALL_ROOT/share/cumbia-project-tools/templates/app
# or $CUMBIA_INSTALL_ROOT/share/cumbia-project-tools/templates/app/python (option -p)
if ! cp -r "$template_dir" "$dest_dir"; then
    echo "Error copying template to $dest_dir"
    exit 1
fi

# Rename widget files
if [[ -z "$pythonpro" ]]; then
    mv "$dest_dir/src/widget.cpp" "$dest_dir/src/${project_name}.cpp"
    mv "$dest_dir/src/widget.h" "$dest_dir/src/${project_name}.h"
    mv "$dest_dir/src/widget.ui" "$dest_dir/src/${project_name}.ui"
else
    mv "$dest_dir/form.ui" "$dest_dir/${project_name}.ui"
    mv "$dest_dir/widget.py" "$dest_dir/${project_name}.py"
    mv "$dest_dir/widget.pyproject" "$dest_dir/${project_name}.pyproject"
fi

# Function to replace placeholders in files
replace_placeholders() {
    local file=$1
    sed -i "s/\${project}/$project_name/g" "$file"
    sed -i "s/\${class_name}/$class_name/g" "$file"
    sed -i "s/\${author}/$author/g" "$file"
    sed -i "s/\${mail}/$mail/g" "$file"
    sed -i "s/\${phone}/$phone/g" "$file"
    sed -i "s/\${organization}/$organization/g" "$file"
    sed -i "s/\${location}/$location/g" "$file"
    sed -i "s/\${spokesperson}/$spokesperson/g" "$file"
}

# Recursively replace placeholders in all files
find "$dest_dir" -type f | while read -r file; do
    replace_placeholders "$file"
done

if find "$dest_dir" -name ".gitignore" -print -quit | grep -q .; then
    echo -e "\e[1;36m-\e[0m directory \e[1;3m$dest_dir\e[0m contains \e[1;3m.gitignore\e[0m file"
fi

# Success message
echo ""
if [[ -z "$pythonpro" ]]; then
    echo -e "\e[1;36m-\e[0m move into \e[1;3m$dest_dir\e[0m:"
    echo -e "\e[1;3m  cd \"$dest_dir\"\e[0m"
    echo -e "\n\e[1;32m*\e[0m project $project_name created successfully in \e[1;3m$dest_dir\e[0m\n"
else
    echo -e "\e[1;36m?\e[0m testing python environment for \e[1;3mpycumbia\e[0m:"

    echo ""
    # function sourced from pycumbia-python-test.sh
    python_test
    echo ""

    echo -e "\e[1;36m-\e[0m move into \e[1;3m$dest_dir\e[0m:"
    echo -e "\e[1;3m  cd \"$dest_dir\"\e[0m"
    echo -e "\e[1;36m-\e[0m run the following command to regenerate the \e[1;3mui_$project_name.py\e[0m file whenever you modify $project_name.ui:"
    echo -e "\e[1;3m  pyside6-uic $project_name.ui -o ui_$project_name.py\e[0m\n"

    echo -e "\e[1;32m*\e[0m python project $project_name created successfully in \e[1;3m$dest_dir\e[0m"
fi
echo ""

if [[ "$remote" == true ]]; then
    cd $dest_dir

    if [[ -n "$group" ]]; then
        cumbia-project-git.sh -c -p $project_name -g $group
        res=$?
        origin=$gitlab_url/$group/$project_name.git
    else
        cumbia-project-git.sh -c -p $project_name
        res=$?
        origin=$gitlab_url/$gitlab_user/$project_name.git
    fi

    if [[ $res -eq 0 ]]; then
        cd $dest_dir
        if [[ $? -eq 0 ]]; then
            git init --initial-branch=master
        fi
        if [[ $? -eq 0 ]]; then
            git remote add origin $origin
        fi
        if [[ $? -eq 0 ]]; then
            git add .
        fi
        if [[ $? -eq 0 ]]; then
            git commit -m "$project_name initial commit"
        fi
        if [[ $? -eq 0 ]]; then
            git push --set-upstream origin master
        fi
    fi

    echo "cumbia-project-git.sh exited with $res"
fi
