import os
import re

def find_files(project_dir):
    """Trova i file main.cpp, myproject.cpp e myproject.h nella directory src/"""
    files = {'main': None, 'cpp': None, 'h': None}

    src_dir = os.path.join(project_dir, 'src')
    for root, _, filenames in os.walk(src_dir):
        for filename in filenames:
            if filename == 'main.cpp':
                files['main'] = os.path.join(root, filename)
            elif filename.endswith('.cpp') and 'main' not in filename:
                files['cpp'] = os.path.join(root, filename)
            elif filename.endswith('.h'):
                files['h'] = os.path.join(root, filename)

    if not all(files.values()):
        raise FileNotFoundError("Impossibile trovare i file necessari: main.cpp, .cpp e .h")

    return files

def detect_class_name(file_path):
    """Rileva automaticamente il nome della classe in un file .cpp o .h"""
    with open(file_path, 'r') as file:
        content = file.read()
        match = re.search(r'class\s+(\w+)', content)
        if match:
            return match.group(1)
        else:
            raise ValueError(f"Classe non trovata nel file {file_path}")

def update_main_cpp(file_path):
    """Applica le sostituzioni richieste in main.cpp"""
    with open(file_path, 'r') as file:
        content = file.read()

    # Sostituisci la riga di QuApplication
    content = re.sub(
        r'QuApplication\s+qu_app\s*\(\s*argc,\s*argv\s*\);',
        '''CumbiaPool *p = new CumbiaPool();
CuControlsFactoryPool fpool;
CuEngineAccessor *a = new CuEngineAccessor(&qu_app, &p, &fpool);
QuApplication qu_app( argc, argv, p, &fpool);''',
        content
    )

    # Sostituisci CuEngineAccessor *c = w->findChild<CuEngineAccessor *>()
    content = re.sub(
        r'CuEngineAccessor\s*\*\s*c\s*=\s*w->findChild<CuEngineAccessor\s*\*>.*;',
        'CuEngineAccessor *c = a;',
        content
    )

    with open(file_path, 'w') as file:
        file.write(content)

    print(f"{file_path} aggiornato.")

def update_header_file(file_path, class_name):
    """Modifica il costruttore e rimuove variabili non necessarie nel file .h"""
    with open(file_path, 'r') as file:
        content = file.readlines()

    constructor_pattern = re.compile(rf'\b{class_name}\s*\(\s*.*\s*\)')
    pool_pattern = re.compile(r'(CumbiaPool|CuControlsFactoryPool)')
    log_pattern = re.compile(r'QuLogImpl')

    modified_content = []
    for line in content:
        if constructor_pattern.search(line):
            modified_content.append(f'    {class_name}(QWidget *parent);\n')  # Costruttore semplificato
        elif pool_pattern.search(line):
            continue  # Salta dichiarazioni di CumbiaPool e CuControlsFactoryPool
        elif log_pattern.search(line):
            modified_content.append(line)  # Mantieni QuLogImpl
        else:
            modified_content.append(line)

    with open(file_path, 'w') as file:
        file.writelines(modified_content)

    print(f"{file_path} aggiornato.")

def update_cpp_file(file_path, class_name):
    """Modifica il costruttore e aggiunge nuove righe nel file .cpp"""
    with open(file_path, 'r') as file:
        content = file.readlines()

    constructor_pattern = re.compile(rf'{class_name}::\s*{class_name}\s*\(.*\)')
    module_loader_pattern = re.compile(r'CuModuleLoader\s+.*')
    setup_ui_pattern = re.compile(r'ui->setupUi\(.*\);')

    modified_content = []
    inside_constructor = False
    constructor_body_added = False

    for line in content:
        if constructor_pattern.search(line):
            modified_content.append(f'{class_name}::{class_name}(QWidget *parent)\n')
            inside_constructor = True
        elif inside_constructor and not constructor_body_added:
            modified_content.append('{\n')
            modified_content.append('    QuApplication *a = static_cast<QuApplication *>(QCoreApplication::instance());\n')
            modified_content.append('    CuModuleLoader mloader(a->cumbiaPool(), a->fpool(), &m_log_impl);\n')
            constructor_body_added = True
        elif module_loader_pattern.search(line):
            continue  # Salta l'esistente dichiarazione di CuModuleLoader
        elif setup_ui_pattern.search(line):
            modified_content.append('    ui->setupUi(this);\n')  # Semplifica setupUi
        else:
            modified_content.append(line)

    with open(file_path, 'w') as file:
        file.writelines(modified_content)

    print(f"{file_path} aggiornato.")

# Directory del progetto
project_dir = os.getcwd()

# Trova i file necessari
files = find_files(project_dir)

# Rileva il nome della classe
print("cpp file " , files['cpp'])
class_name = detect_class_name(files['h'])

# Aggiorna i file
update_main_cpp(files['main'])
update_header_file(files['h'], class_name)
update_cpp_file(files['cpp'], class_name)

print("Aggiornamento completato.")
