#!/bin/bash

if [[ -z "$CUMBIA_INSTALL_ROOT" ]]; then
    echo -e "[\e[1;31mNO\e[0m]: CUMBIA_INSTALL_ROOT not exported. Please"
    echo -e "export CUMBIA_INSTALL_ROOT=path/to/cumbia-libs"
    exit 1
fi

gitlab_groups_enabled() {
    if [[ ${#gitlab_groups_search[@]} -eq 0 ]]; then
        hints+=("add gitlab_groups_search=(\"cs/gui\" \"cs/lib\" \"cs/lib/cumbia\")" \
            "line into $CONFIG_FILE" \
            "if you want to search within specific groups for the project in gitlab")
        return 1
    else
        return 0
fi
}

gitlab_enabled() {
    if [[ -n "$gitlab_token" ]] && [[ -n "$gitlab_url" ]] && which jq > /dev/null 2>&1; then
        return 0
    fi
    return 1
}

check_gitlab_config() {
    if [[ -z "$gitlab_token" ]]; then
        echo -e "add gitlab_token=your-gitlab-personal-access-token"
        echo -e "line into $CONFIG_FILE"
        echo -e "if you want to search for the project in gitlab"
        exit 1
    fi
    if [[ -z "$gitlab_url" ]]; then
        echo -e "add gitlab_url=\"https://gitlab.elettra.eu\""
        echo -e "line into $CONFIG_FILE"
        echo -e "if you want to search for the project in gitlab"
        exit 1
    fi
    if ! which jq > /dev/null 2>&1; then
        echo -e "[\e[1;31mNO\e[0m]: jq is not installed. Please install jq"
        echo -e "if you want to search for the project in gitlab"
    exit 1
    fi
}
# Function to list all development directories (-L)
list_dev_dirs() {
    for dir in "${dev_dirs[@]}"; do
        echo "$dir"
    done
}

# Function to list projects with optional search (-l with -s)
list_projects() {
    if [[ -z "$dir_leaf" ]]; then
        # No specific directory leaf provided, list projects from all configured directories
        for dir in "${dev_dirs[@]}"; do
            for proj in "$dir"/*; do
                if [[ -d "$proj" ]]; then
                    proj_name=$(basename "$proj")
                    if [[ -n "$search_str" && "$proj_name" != *"$search_str"* ]]; then
                        continue
                    fi

                    if [[ -f "$proj/CMakeLists.txt" ]]; then
                        echo -e "$proj_name [\e[1;32mcmake\e[0m]"
                    elif [[ -n $(find "$proj" -maxdepth 1 -name '*.pro') ]]; then
                        echo -e "$proj_name [\e[1;36mqmake\e[0m]"
                    else
                        echo -e "$proj_name [\e[1;35munknown\e[0m]"
                    fi
                fi
            done
        done
    else
        # Specific directory leaf provided, list projects from the matched directory
        local target_dir=""
        for dir in "${dev_dirs[@]}"; do
            if [[ "$(basename "$dir")" == "$dir_leaf" ]]; then
                target_dir="$dir"
                break
            fi
        done

        if [[ -z "$target_dir" ]]; then
            echo "Directory leaf '$dir_leaf' not found."
            exit 1
        fi

        echo "Projects in $target_dir:"
        for proj in "$target_dir"/*; do
            if [[ -d "$proj" ]]; then
                proj_name=$(basename "$proj")
                if [[ -n "$search_str" && "$proj_name" != *"$search_str"* ]]; then
                    continue
                fi

                if [[ -f "$proj/CMakeLists.txt" ]]; then
                    echo "$proj_name [\e[1;32mcmake\e[0m]"
                elif [[ -n $(find "$proj" -maxdepth 1 -name '*.pro') ]]; then
                    echo "$proj_name [\e[1;36mqmake\e[0m]"
                else
                    echo "$proj_name [\e[1;35munknown\e[0m]"
                fi
            fi
        done
    fi
}

# Function to get the group ID from the GitLab group name
get_group_id() {
    local group_name="$1"

    # Fetch the group information using the GitLab API
    group_info=$(curl -s --header "PRIVATE-TOKEN: $gitlab_token" "$gitlab_url/api/v4/groups?search=$group_name")
    # Extract the group ID using jq
    group_id=$(echo "$group_info" |jq -r --arg full_path "$group_name" '.[] | select(.full_path == $full_path) | .id')

    if [[ -z "$group_id"  ]] || [[ "$group_id" == "null" ]]; then
        echo "Error: Group '$group_name' not found."
        return 1
    fi

    echo "$group_id"
}

gitlab_group_search() {
    local project="$1"
    local gro="$2"
    if [[ -n "$gro" ]]; then
        group_id=$(get_group_id "$gro")
        if [[ $? -ne 0 ]]; then
            echo -e "[\e[1;31mNO\e[0m]: Failed to retrieve group ID."
            exit 1
        fi
        while :; do
            response=$(curl --silent --header "PRIVATE-TOKEN: $gitlab_token" \
                "$gitlab_url/api/v4/groups/$group_id/search?scope=projects&search=$project&per_page=100")
                # Check if there's another page

            projects=$(echo "$response" | sed -n '/^\[/,$p')  # Extract JSON part (starting from '[')
            next_page=$(echo "$response" | grep -i "X-Next-Page" | awk '{print $2}' | tr -d '\r')

            if [[ $(echo "$projects" | jq 'length') -gt 0 ]]; then
                echo "$projects"
                break
            fi
            # Exit if no next page
            if [[ -z "$next_page" ]]; then
                break
            fi
            page="$next_page"
        done
    else
        for group in "${gitlab_groups_search[@]}"; do

            group_id=$(get_group_id "$group")
            if [[ $? -ne 0 ]]; then
                echo -e "[\e[1;31mNO\e[0m]: Failed to retrieve group ID."
                exit 1
            fi
            local page=1
            # curl --header "PRIVATE-TOKEN: <your_access_token>" \
            # "https://gitlab.example.com/api/v4/groups/3/   \
            # search?scope=projects&search=flight"


            while :; do
                response=$(curl --silent --header "PRIVATE-TOKEN: $gitlab_token" \
                    "$gitlab_url/api/v4/groups/$group_id/search?scope=projects&search=$project&per_page=100&page=$page")

                projects=$(echo "$response" | sed -n '/^\[/,$p')  # Extract JSON part (starting from '[')
                next_page=$(echo "$response" | grep -i "X-Next-Page" | awk '{print $2}' | tr -d '\r')


                if [[ $(echo "$projects" | jq 'length') -gt 0 ]]; then
                    echo "$projects"
                    break
                fi
#                echo "executing curl --silent --header \"PRIVATE-TOKEN: $gitlab_token\" \
#                    \"$gitlab_url/api/v4/groups/$group_id/search?scope=projects&search=$project&per_page=100&page=$page\"" >&2
                # Exit if no next page
                if [[ -z "$next_page" ]]; then
                    break
                fi
                page="$next_page"

                echo -e "projects: \e[1;32m$projects\e[0m\n" >&2
            done
    #       -- debug --
    #       echo "executing \"curl --silent --header \"PRIVATE-TOKEN: $gitlab_token\" \
    #            \"$gitlab_url/api/v4/groups/$group_id/search?scope=projects&search=$project\" "
    #       echo -e "PROJECTS\n$projects\n\n"
        done
    fi
    echo "$projects"
}

json_extract() {
    json="$1"
    search_str="$2"
    exact="$3"

    # Check if the search string contains a wildcard (*)
    if [[ -z "$exact" ]]; then
        # Convert wildcard (*) to a jq-compatible regex (.*)
        search_result=$(echo "$json" | jq -r --arg pattern "$search_str" '.[] | select(.name | test($pattern)) | .http_url_to_repo')
    else
        # No wildcard, search for exact match
        search_result=$(echo "$json" | jq -r --arg name "$search_str" '.[] | select(.name == $name) | .http_url_to_repo')
    fi

    if [[ $? -ne 0 ]]; then
        echo "jq error on \"$json\" search string \"$search_str\""  >&2
    fi

    # Print the search result
    if [[ -n "$search_result" ]]; then
        echo "$search_result"
    fi
}

gitlab_search() {
    projects=$(curl --silent --header "PRIVATE-TOKEN: $gitlab_token" \
        "$gitlab_url/api/v4/search?scope=projects&search=$project&per_page=1000")
    echo "$projects"
}

local_search () {
    local search_str="$1"
    local exact_match=$2
    for dir in "${dev_dirs[@]}"; do
        # Check if the search string is enclosed in double quotes for an exact match
        if [[ -n "$exact_match" ]] && [[ -d $dir ]]; then
            search_result=$(find $dir -maxdepth 1 -mindepth 1 -type d -name "$search_str")
        elif [[ -d $dir ]]; then
            search_result=$(find $dir -maxdepth 1 -mindepth 1 -type d -name "*$search_str*")
        elif [[ ! -d $dir ]]; then
            warns+=("dir \"$dir\" specified in config file $CONFIG_FILE does not exist")
        fi

        # "return" with echo
        if [[ -n "$search_result" ]]; then
            echo "$search_result"
        fi
    done
}

# Function to determine project type (-t)
get_project_type() {
    if [[ -f "$project_path/CMakeLists.txt" ]]; then
        echo "cmake"
    elif [[ -n $(find "$project_path" -maxdepth 1 -name '*.pro') ]]; then
        echo "qmake"
    else
        echo "Unknown project type."
        exit 1
    fi
}

